# This file was auto-generated by salt's setup on Tuesday, 13 October 2015 @ 21:10:17 UTC.

from salt.version import SaltStackVersion

__saltstack_version__ = SaltStackVersion(2014, 7, 7, 0, 0, 0, None)
