body{font-family:"Lato",Helvetica,Arial,sans-serif;background-color:#fff;color:#676767}
.footer{padding-top:50px}
a{color:#428bca;text-decoration:none}
a:hover,a:focus{color:#2a6496;text-decoration:underline}
blockquote{font-size:15px}
button#dropdownMenu1{width:90%;padding:10px;margin:10px}
.panel-default>.panel-heading{background-color:#f8f8f2;border:1px solid #e3e3e3}
.well{background-color:#f5f5f5;border:1px solid #e3e3e3}
.nav>li>a{position:relative;display:block;padding:0}
#guide-links a,#guide-title{text-transform:uppercase;font-weight:300}
#guide-title,.well-title{text-transform:uppercase;font-weight:300}
.body-content img{max-width:100%;width:auto\9;height:auto;vertical-align:middle;border:0;-ms-interpolation-mode:bicubic}
td img{max-width:none}
img.center{display:block;margin:auto}
.jumbotron h1{font-weight:300;text-transform:none}
.jumbotron{background-color:#171717;color:#fff;text-shadow:#000 .1em .1em .1em}
.jumbotron .glyphicon{color:#48b4fb;font-weight:100;padding-right:10px;font-size:smaller}
.panel-title a{font-weight:300}
.panel-heading{padding:20px}
.panel-group a{text-decoration:none}
.list-group-item{border:none;padding-left:25px}
.list-group-item a{color:#333;display:list-item}
h1,h2,h3,h4,h5,h6,.h1,.h2,.h3,.h4,.h5,.h6{text-transform:uppercase}
h1,.h1{color:#48b4fb;font-size:40px;margin-top:10px;margin-bottom:30px}
h2,.h2{color:#48b4fb;font-size:23px}
h3,.h3{font-size:18px}
h4,.h4{font-size:16px}
h5,.h5{font-size:16px;font-style:italic}
.subhead{color:#C0392B;font-size:16px;text-transform:uppercase}
.reference h3{font-size:16pt;line-height:1.9;color:#D75400;font-weight:300}
.head3 h2{font-size:21px;color:#C0392B}
.head4 h2{font-size:16px;color:#C0392B}
.btn-primary{color:#fff;background-color:#424242;border-color:#080808}
.btn-primary:hover{color:#fff;background-color:#424242;border-color:#080808}
.btn-secondary{color:#333;background-color:#f5f5f5;border-color:#ccc}
button#next-button{float:right;margin:15px 0 0}
button#prev-button{float:left;margin:15px 0 0}
.inner-addon{position:relative}
.inner-addon .glyphicon{position:absolute;padding:10px;pointer-events:none}
.glyphicon{color:#C0392B;font-weight:100}
#next-button .glyphicon,#prev-button .glyphicon{color:#48b4fb}
.left-addon .glyphicon{padding:14px 0 0;left:10px}
.right-addon .glyphicon{padding:30px 0 0;right:5px}
.left-addon input{padding-left:30px}
.right-addon input{padding-right:30px}
#search-form{padding:5px 5px 10px}
#wrapper{padding-left:0;-webkit-transition:all .5s ease;-moz-transition:all .5s ease;-o-transition:all .5s ease;transition:all .5s ease}
#wrapper.toggled{padding-left:250px}
#sidebar-wrapper{z-index:1000;position:fixed;top:0;right:260px;width:0;height:100%;margin-right:-250px;overflow-y:hidden;overflow-x:hidden;-webkit-transition:all .5s ease;-moz-transition:all .5s ease;-o-transition:all .5s ease;transition:all .5s ease;background-color:#fff}
#sidebar-nav{position:absolute;top:0;width:260px;font-size:14px}
#wrapper.toggled #sidebar-wrapper{width:260px}
#page-content-wrapper{width:95%;padding:15px;font-size:15px;line-height:1.7}
#wrapper.toggled #page-content-wrapper{position:absolute;margin-left:-250px}
#dev-notification{position:fixed;top:0;left:0;width:100%;background-color:#fff;text-align:center;z-index:20}
.alert.alert-warning.dev-notification-text{padding:0;margin-bottom:0;width:95%}
@media(min-width:768px) {
    #wrapper{padding-right:260px}
    #wrapper.toggled{padding-right:0}
    #sidebar-wrapper{width:260px}
    #wrapper.toggled #sidebar-wrapper{width:0}
    #page-content-wrapper{padding:20px 0 0}
    #wrapper.toggled #page-content-wrapper{position:relative;margin-right:0}
    #menu-toggle{display:none!important}
    .alert.alert-warning.dev-notification-text{width:65%}
}
@media(min-width:992px) {
    #wrapper{padding-right:300px}
    #wrapper.toggled{padding-right:0}
    #sidebar-wrapper{width:300px}
    #sidebar-nav{width:300px}
    #wrapper.toggled #sidebar-wrapper{width:0}
    #page-content-wrapper{padding:20px 0 0}
    #wrapper.toggled #page-content-wrapper{position:relative;margin-right:0}
    #menu-toggle{display:none!important}
}
@media(min-width:1200px) {
    #wrapper{padding-right:350px}
    #sidebar-wrapper{margin-right:-210px}
    #wrapper.toggled{padding-right:0}
    #sidebar-wrapper{width:350px}
    #sidebar-nav{width:300px}
    #wrapper.toggled #sidebar-wrapper{width:0}
    #page-content-wrapper{padding:20px 0 0}
    #wrapper.toggled #page-content-wrapper{position:relative;margin-right:0}
    #menu-toggle{display:none!important}
    .alert.alert-warning.dev-notification-text{width:68%}
}
@media(min-width:1400px) {
    #sidebar-wrapper{margin-right:-175px}
    #menu-toggle{display:none!important}
    .alert.alert-warning.dev-notification-text{width:70%}
}
@media(max-width:768px) {
    #sidebar-wrapper{border:1px solid #424242}
}
#header-nav{float:right;text-align:right}
#header-nav li{padding:0 0 15px 15px;font-weight:300}
#sidebar-wrapper{visibility:hidden}
#sidebar-nav ul{list-style:none;margin:0;padding:0}
#sidebar-nav .collapsed > ul{display:none}
#sidebar-nav .glyphicon{padding-left:5px;font-size:14px}
.nav>li>a:hover,.nav>li>a:focus{background-color:transparent}
#sidebar-nav li a{display:block;text-decoration:none;color:#424242;border-top:1px solid #e3e3e3;font-size:12px}
#sidebar-nav ul li:not(.tocify-item){padding-left:10px}
#sidebar-nav ul a{padding-top:5px;padding-bottom:5px}
#sidebar-nav > ul > .current{min-height:20px;padding:5px;background-color:#f5f5f5;border:1px solid #e3e3e3;border-radius:4px;-webkit-box-shadow:inset 0 1px 1px rgba(0,0,0,.05);box-shadow:inset 0 1px 1px rgba(0,0,0,.05)}
#sidebar-nav > ul > .current a{border:none}
#sidebar-nav .tocify li.current > a{font-weight:400;background:#48b4fb}
#sidebar-nav .tocify ul{list-style:circle}
#sidebar-nav .tocify li a{font-weight:400;font-size:14px;background-color:#34495E;padding-left:4px}
.tocify li a{display:block;text-decoration:none;color:#fff;border-bottom:none;font-size:14px;font-weight:300}
#tocify-header0 > li a{border-bottom:1px solid #2C3E50}
ul.tocify-subheader ul li{margin-left:20px}
#gen-links > li.current > .tocify > ul > li > a{font-weight:400;font-size:20px;background:transparent;border-bottom:none;border-top:none;color:#333}
#sidebar-nav li.current:not(.tocify-item) > a{font-weight:400;font-size:20px;border-bottom:none;border-top:none}
#gen-links{border-bottom:1px solid #e3e3e3}
#guide-links a{border-top:none;font-size:20px;color:#333}
#guide-links a:hover,a:focus{text-decoration:none}
li.current + li > a{border-top:none}
.reference.nested1{padding:5px;margin-bottom:40px;background-color:#FDFDFD;border:1px solid #e3e3e3;border-radius:4px;-webkit-box-shadow:inset 0 1px 1px rgba(0,0,0,.05);box-shadow:inset 0 1px 1px rgba(0,0,0,.05)}
tr.sthead.prophead{color:#34495E;text-transform:uppercase}
aside{min-height:20px;padding:5px;border:1px solid #eee;background:rgba(255,255,255,0.5);border-radius:4px;width:95%;margin:0 auto;margin-bottom:20px;padding-left:50px;font-weight:300}
aside .glyphicon{font-size:20pt;float:left;margin-right:15px;height:100%;margin-left:-40px}
aside .title{text-transform:uppercase;color:#34495E}
aside .glyphicon{color:#F4C600}
.important .title{color:#D75400;font-weight:600}
.versionmodified{color:#009276}
#header-links{text-align:center;font-weight:300;padding-top:15px}
.versions{text-align:center;font-weight:300;border-top:1px solid #e3e3e3;border-bottom:1px solid #e3e3e3;padding:5px}
.versions a:hover,a:focus{text-decoration:none}
.permalink i{color:#fff;font-size:14pt;padding-left:5px}
.permalink i:hover{color:#000}
li.toctree-l3.current ul{margin-left:15px}
li.toctree-l2.current ul{margin-left:15px}
li.toctree-l1.current ul{margin-left:15px}
.nav.nav-center{margin:0;float:none}
ul.nav.collapsed{margin-left:80px;font-size:14pt}
#expanded-nav{font-size:12pt;font-weight:300}
#expanded-nav li{padding:0 15px 0 0}
ul.nav.collapsed li{padding:0 0 15px 10px;margin:0}
#social-links{text-align:center;padding:10px 0}
.navbar-default{background-color:#fff;border-color:#fff}
.navbar-header{width:100%}
.open>.dropdown-menu{width:100%}
#lnav-title{font-weight:300;font-size:16pt;text-align:center}
pre{border-top-width:4px;padding-top:18px}
code{font-family:Monaco,Menlo,Consolas,"Courier New",monospace;color:#333;-webkit-border-radius:3px;-moz-border-radius:3px;border-radius:3px;white-space:nowrap;background-color:rgba(0,0,0,.1)}
p code{font-size:12px}
code.xref{color:#4070a0}
code.descclassname{background-color:transparent;border:none}
code.descname{background-color:transparent;font-weight:700;font-size:1.5em}
dl.function{margin-bottom:30px;border-bottom:1px solid #eee}
#sidebar-nav li a.selected{font-weight:700}
#sidebar-nav a.current.reference.internal{background-color:#424242;color:#fff;padding:2px 4px;-webkit-border-radius:3px;-moz-border-radius:3px;border-radius:3px}
#sidebar-nav a.current code{color:#fff;font-size:14pt}
.intro-text{font-size:18pt;line-height:1.3;font-weight:300}
img.logo{margin-top:-10px}
.notifications{float:right;font-size:13pt}
div.logo{background-color:#000}
.modal-dialog{float:right;padding:125px 10px 0 0;width:375px}
.modal-backdrop.in{opacity:0}
.modal{direction:rtl;overflow-y:auto}
.modal .modal-dialog{direction:ltr}
.modal-open{overflow:auto}
.bootbox-body h3{color:#48b4fb}
.bootbox-body h3:first-child{margin-top:5px}
div.versions .btn{padding: 3px 6px}
div.versions p{margin: 3px 0 0}
div.versions a,div.versions a:hover,div.versions a:active{color: #428bca}

/*pygments*/

.highlight .hll { background-color: #ffffcc }
.highlight  { background: #eeffcc; }
.highlight .c { color: #408090; font-style: italic } /* Comment */
.highlight .err { border: 1px solid #FF0000 } /* Error */
.highlight .k { color: #007020; font-weight: bold } /* Keyword */
.highlight .o { color: #666666 } /* Operator */
.highlight .cm { color: #408090; font-style: italic } /* Comment.Multiline */
.highlight .cp { color: #007020 } /* Comment.Preproc */
.highlight .c1 { color: #408090; font-style: italic } /* Comment.Single */
.highlight .cs { color: #408090; background-color: #fff0f0 } /* Comment.Special */
.highlight .gd { color: #A00000 } /* Generic.Deleted */
.highlight .ge { font-style: italic } /* Generic.Emph */
.highlight .gr { color: #FF0000 } /* Generic.Error */
.highlight .gh { color: #000080; font-weight: bold } /* Generic.Heading */
.highlight .gi { color: #00A000 } /* Generic.Inserted */
.highlight .go { color: #333333 } /* Generic.Output */
.highlight .gp { color: #c65d09; font-weight: bold } /* Generic.Prompt */
.highlight .gs { font-weight: bold } /* Generic.Strong */
.highlight .gu { color: #800080; font-weight: bold } /* Generic.Subheading */
.highlight .gt { color: #0044DD } /* Generic.Traceback */
.highlight .kc { color: #007020; font-weight: bold } /* Keyword.Constant */
.highlight .kd { color: #007020; font-weight: bold } /* Keyword.Declaration */
.highlight .kn { color: #007020; font-weight: bold } /* Keyword.Namespace */
.highlight .kp { color: #007020 } /* Keyword.Pseudo */
.highlight .kr { color: #007020; font-weight: bold } /* Keyword.Reserved */
.highlight .kt { color: #902000 } /* Keyword.Type */
.highlight .m { color: #208050 } /* Literal.Number */
.highlight .s { color: #4070a0 } /* Literal.String */
.highlight .na { color: #4070a0 } /* Name.Attribute */
.highlight .nb { color: #007020 } /* Name.Builtin */
.highlight .nc { color: #0e84b5; font-weight: bold } /* Name.Class */
.highlight .no { color: #60add5 } /* Name.Constant */
.highlight .nd { color: #555555; font-weight: bold } /* Name.Decorator */
.highlight .ni { color: #d55537; font-weight: bold } /* Name.Entity */
.highlight .ne { color: #007020 } /* Name.Exception */
.highlight .nf { color: #06287e } /* Name.Function */
.highlight .nl { color: #002070; font-weight: bold } /* Name.Label */
.highlight .nn { color: #0e84b5; font-weight: bold } /* Name.Namespace */
.highlight .nt { color: #062873; font-weight: bold } /* Name.Tag */
.highlight .nv { color: #bb60d5 } /* Name.Variable */
.highlight .ow { color: #007020; font-weight: bold } /* Operator.Word */
.highlight .w { color: #bbbbbb } /* Text.Whitespace */
.highlight .mb { color: #208050 } /* Literal.Number.Bin */
.highlight .mf { color: #208050 } /* Literal.Number.Float */
.highlight .mh { color: #208050 } /* Literal.Number.Hex */
.highlight .mi { color: #208050 } /* Literal.Number.Integer */
.highlight .mo { color: #208050 } /* Literal.Number.Oct */
.highlight .sb { color: #4070a0 } /* Literal.String.Backtick */
.highlight .sc { color: #4070a0 } /* Literal.String.Char */
.highlight .sd { color: #4070a0; font-style: italic } /* Literal.String.Doc */
.highlight .s2 { color: #4070a0 } /* Literal.String.Double */
.highlight .se { color: #4070a0; font-weight: bold } /* Literal.String.Escape */
.highlight .sh { color: #4070a0 } /* Literal.String.Heredoc */
.highlight .si { color: #70a0d0; font-style: italic } /* Literal.String.Interpol */
.highlight .sx { color: #c65d09 } /* Literal.String.Other */
.highlight .sr { color: #235388 } /* Literal.String.Regex */
.highlight .s1 { color: #4070a0 } /* Literal.String.Single */
.highlight .ss { color: #517918 } /* Literal.String.Symbol */
.highlight .bp { color: #007020 } /* Name.Builtin.Pseudo */
.highlight .vc { color: #bb60d5 } /* Name.Variable.Class */
.highlight .vg { color: #bb60d5 } /* Name.Variable.Global */
.highlight .vi { color: #bb60d5 } /* Name.Variable.Instance */
.highlight .il { color: #208050 } /* Literal.Number.Integer.Long */

/*basic*/

div.clearer{clear:both}
div.related{width:100%;font-size:90%}
div.related h3{display:none}
div.related ul{margin:0;padding:0 0 0 10px;list-style:none}
div.related li{display:inline}
div.related li.right{float:right;margin-right:5px}
div.sphinxsidebarwrapper{padding:10px 5px 0 10px}
div.sphinxsidebar{float:left;width:230px;margin-left:-100%;font-size:90%}
div.sphinxsidebar ul{list-style:none}
div.sphinxsidebar ul ul,div.sphinxsidebar ul.want-points{margin-left:20px;list-style:square}
div.sphinxsidebar ul ul{margin-top:0;margin-bottom:0}
div.sphinxsidebar form{margin-top:10px}
div.sphinxsidebar input{border:1px solid #98dbcc;font-family:sans-serif;font-size:1em}
div.sphinxsidebar #searchbox input[type="text"]{width:170px}
div.sphinxsidebar #searchbox input[type="submit"]{width:30px}
img{border:0;max-width:100%}
ul.search{margin:10px 0 0 20px;padding:0}
ul.search li{padding:5px 0 5px 20px;background-image:url(file.png);background-repeat:no-repeat;background-position:0 7px}
ul.search li a{font-weight:700}
ul.search li div.context{color:#888;margin:2px 0 0 30px;text-align:left}
ul.keywordmatches li.goodmatch a{font-weight:700}
table.contentstable{width:90%}
table.contentstable p.biglink{line-height:150%}
a.biglink{font-size:1.3em}
span.linkdescr{font-style:italic;padding-top:5px;font-size:90%}
table.indextable{width:100%}
table.indextable td{text-align:left;vertical-align:top}
table.indextable dl,table.indextable dd{margin-top:0;margin-bottom:0}
table.indextable tr.pcap{height:10px}
table.indextable tr.cap{margin-top:10px;background-color:#f2f2f2}
img.toggler{margin-right:3px;margin-top:3px;cursor:pointer}
div.modindex-jumpbox{border-top:1px solid #ddd;border-bottom:1px solid #ddd;margin:1em 0;padding:.4em}
div.genindex-jumpbox{border-top:1px solid #ddd;border-bottom:1px solid #ddd;margin:1em 0;padding:.4em}
a.headerlink{visibility:hidden}
h1:hover > a.headerlink,h2:hover > a.headerlink,h3:hover > a.headerlink,h4:hover > a.headerlink,h5:hover > a.headerlink,h6:hover > a.headerlink,dt:hover > a.headerlink,caption:hover > a.headerlink,p.caption:hover > a.headerlink,div.code-block-caption:hover > a.headerlink{visibility:visible}
div.body p.caption{text-align:inherit}
div.body td{text-align:left}
.field-list ul{padding-left:1em}
.first{margin-top:0!important}
p.rubric{margin-top:30px;font-weight:700}
img.align-left,.figure.align-left,object.align-left{clear:left;float:left;margin-right:1em}
img.align-right,.figure.align-right,object.align-right{clear:right;float:right;margin-left:1em}
img.align-center,.figure.align-center,object.align-center{display:block;margin-left:auto;margin-right:auto}
.align-left{text-align:left}
.align-center{text-align:center}
.align-right{text-align:right}
div.sidebar{margin:0 0 .5em 1em;border:1px solid #ddb;padding:7px 7px 0;background-color:#ffe;width:40%;float:right}
p.sidebar-title{font-weight:700}
div.topic{border:1px solid #ccc;padding:7px 7px 0;margin:10px 0}
p.topic-title{font-size:1.1em;font-weight:700;margin-top:10px}
div.admonition{margin-top:10px;margin-bottom:10px;padding:7px}
div.admonition dt{font-weight:700}
div.admonition dl{margin-bottom:0}
p.admonition-title{margin:0 10px 5px 0;font-weight:700}
div.body p.centered{text-align:center;margin-top:25px}
table.docutils{border:0;border-collapse:collapse}
table caption span.caption-number{font-style:italic}
table.docutils td,table.docutils th{padding:1px 8px 1px 5px;border-top:0;border-left:0;border-right:0;border-bottom:1px solid #aaa}
table.field-list td,table.field-list th{border:0!important}
table.footnote td,table.footnote th{border:0!important}
th{text-align:left;padding-right:5px}
table.citation{border-left:solid 1px gray;margin-left:1px}
table.citation td{border-bottom:none}
div.figure{margin:.5em;padding:.5em}
div.figure p.caption{padding:.3em}
div.figure p.caption span.caption-number{font-style:italic}
ol.arabic{list-style:decimal}
ol.loweralpha{list-style:lower-alpha}
ol.upperalpha{list-style:upper-alpha}
ol.lowerroman{list-style:lower-roman}
ol.upperroman{list-style:upper-roman}
dl{margin-bottom:15px}
dd p{margin-top:0}
dd ul,dd table{margin-bottom:10px}
dd{margin-top:3px;margin-bottom:10px;margin-left:30px}
dl.glossary dt{font-weight:700;font-size:1.1em}
.field-list ul{margin:0;padding-left:1em}
.field-list p{margin:0}
.optional{font-size:1.3em}
.sig-paren{font-size:larger}
.versionmodified{font-style:italic}
.system-message{background-color:#fda;padding:5px;border:3px solid red}
.footnote:target{background-color:#ffa}
.line-block{display:block;margin-top:1em;margin-bottom:1em}
.line-block .line-block{margin-top:0;margin-bottom:0;margin-left:1.5em}
.guilabel,.menuselection{font-family:sans-serif}
.accelerator{text-decoration:underline}
.classifier{font-style:oblique}
abbr,acronym{border-bottom:dotted 1px;cursor:help}
pre{overflow:auto;overflow-y:hidden}
td.linenos pre{padding:5px 0;border:0;background-color:transparent;color:#aaa}
table.highlighttable{margin-left:.5em;width:100%}
td.code{width:100%}
table.highlighttable td{padding:0 .5em}
div.code-block-caption{padding:2px 5px;font-size:small}
div.code-block-caption code{background-color:transparent}
div.code-block-caption + div > div.highlight > pre{margin-top:0}
div.code-block-caption span.caption-number{padding:.1em .3em;font-style:italic}
div.literal-block-wrapper{padding:1em 1em 0}
div.literal-block-wrapper div.highlight{margin:0}
h1 code,h2 code,h3 code,h4 code,h5 code,h6 code{background-color:transparent}
.viewcode-link{float:right}
.viewcode-back{float:right;font-family:sans-serif}
div.viewcode-block:target{margin:-1px -10px;padding:0 10px}
img.math{vertical-align:middle}
div.body div.math p{text-align:center}
span.eqno{float:right}
@media print {
div.document,div.documentwrapper,div.bodywrapper{margin:0!important;width:100%}
div.sphinxsidebar,div.related,div.footer,#top-link{display:none}
}

/*main*/

h1:hover > .headerlink,h2:hover > .headerlink,h3:hover > .headerlink,h4:hover > .headerlink,h5:hover > .headerlink,h6:hover > .headerlink,dt:hover > .headerlink{visibility:visible}
.sidebar .toctree-l1.current a{border-right:5px solid #fcaf3e}
.line-block{display:block;margin-top:1em;margin-bottom:1em}
.line-block .line-block{margin-top:0;margin-bottom:0;margin-left:1.5em}
div.header div.rel a{color:#fcaf3e;letter-spacing:.1em;text-transform:uppercase}
.descname{font-weight:700}
blockquote{margin:1em}
div.admonition{font-size:.9em;margin:1em 0;padding:.5em 1em;border:1px solid #ddd}
div.admonition p.admonition-title{font-weight:700;color:#3465a4}
div.warning{border-color:#940000}
div.warning p.admonition-title{color:#940000}
div.viewcode-block:target{background-color:#f4debf;border-top:1px solid #ac9;border-bottom:1px solid #ac9}
.lit-docs{list-style-type:none;margin:0}
.lit-item{padding-bottom:2em}
.lit-annotation,.lit-content{display:inline-block;vertical-align:top}
.lit-annotation{width:40.425531914894%}
.lit-content{width:57.446808510638%}
.lit-annotation pre,.lit-content pre{word-break:normal;word-wrap:normal;white-space:pre;overflow-x:auto}
.lit-content pre{border:none}
.lit-container{position:relative}
.lit-background{position:absolute;top:-1px;bottom:0;right:1.1em;width:57.446808510638%;z-index:-1;background-color:#f5f5f5;border:1px solid #e5e5ee;border:1px solid rgba(0,0,0,0.15);-webkit-border-radius:4px;-moz-border-radius:4px;border-radius:4px}
.scrollable.container{width:auto}
.scrollable{overflow-x:auto;position:relative}
.highlight-bash,.highlight-jinja,.highlight-python,.highlight-yaml{position:relative}
.highlight-bash:before,.highlight-jinja:before,.highlight-python:before,.highlight-yaml:before{z-index:10;font-size:9px;padding:.1em .8em;text-align:center;color:#999;display:block;position:absolute;border-radius:0 3px 0 3px;border-top:none;border-right:none;background-color:#f1f1f1;top:0;right:0;height:18px}
.highlight-bash pre{border-color:#999}
.highlight-jinja pre{border-color:#e00a1b}
.highlight-python pre{border-color:#3377b0}
.highlight-yaml pre{border-color:#ccc}
.highlight-bash:before{content:'BASH';background-color:#444;color:#fff}
.highlight-jinja:before{content:'JINJA';background-color:#e00a1b;color:#fff}
.highlight-python:before{content:'PYTHON';background-color:#3377b0;color:#ffe243}
.highlight-yaml:before{content:'YAML';background-color:#ccc;color:#000}
